"""Annotate dictionaries

Created by:
EOLE (http://eole.orion.education.fr)
Copyright (C) 2005-2018

Forked by:
Cadoles (http://www.cadoles.com)
Copyright (C) 2019-2021

distribued with GPL-2 or later license

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""
from .group import GroupAnnotator
from .service import ServiceAnnotator
from .variable import VariableAnnotator, CONVERT_OPTION
from .check import CheckAnnotator
from .value import ValueAnnotator
from .condition import ConditionAnnotator
from .fill import FillAnnotator
from .family import FamilyAnnotator, modes
from .property import PropertyAnnotator

class SpaceAnnotator:  # pylint: disable=R0903
    """Transformations applied on a CreoleObjSpace instance
    """
    def __init__(self, objectspace, eosfunc_file):
        self.objectspace = objectspace
        GroupAnnotator(objectspace)
        ServiceAnnotator(objectspace)
        VariableAnnotator(objectspace)
        CheckAnnotator(objectspace,
                       eosfunc_file,
                       )
        ConditionAnnotator(objectspace)
        FillAnnotator(objectspace,
                      eosfunc_file,
                      )
        ValueAnnotator(objectspace)
        FamilyAnnotator(objectspace)
        PropertyAnnotator(objectspace)


__all__ = ('SpaceAnnotator', 'CONVERT_OPTION', 'modes')
