"""Rougail's tools

Created by:
EOLE (http://eole.orion.education.fr)
Copyright (C) 2005-2018

Forked by:
Cadoles (http://www.cadoles.com)
Copyright (C) 2019-2021

distribued with GPL-2 or later license

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""
from typing import List
from unicodedata import normalize, combining

from importlib.machinery import SourceFileLoader
from importlib.util import spec_from_loader, module_from_spec


def normalize_family(family_name: str) -> str:
    """replace space, accent, uppercase, ... by valid character
    """
    family_name = family_name.replace('-', '_').replace(' ', '_').replace('.', '_')
    nfkd_form = normalize('NFKD', family_name)
    family_name = ''.join([c for c in nfkd_form if not combining(c)])
    return family_name.lower()


def load_modules(eosfunc_file) -> List[str]:
    """list all functions in eosfunc
    """
    loader = SourceFileLoader('eosfunc', eosfunc_file)
    spec = spec_from_loader(loader.name, loader)
    eosfunc = module_from_spec(spec)
    loader.exec_module(eosfunc)
    return eosfunc
